// Xiphias3
// December 2009

#include "X3RichTextPrinter.h"

#define MAX_RICH_TEXT 255

HMODULE			   m_hModule;
X3RichTextPrinter* m_pRTPArr[MAX_RICH_TEXT];

UINT    __stdcall RTP_Initialize();
BOOL    __stdcall RTP_Terminate(UINT uiID);
VOID    __stdcall RTP_SetText(UINT uiID, LPCWSTR lpwszText);
VOID    __stdcall RTP_SizeText(UINT uiID, HDC hDC, RECT* rect);
BOOL    __stdcall RTP_DrawText(UINT uiID, HDC hDC, RECT* rect);
HRESULT __stdcall RTP_SendMessage(UINT uiID, UINT msg, WPARAM wparam, LPARAM lparam, LRESULT* plresult);
BOOL    __stdcall RTP_DrawTextL(UINT uiID, HDC hDC, LONG lX, LONG lY, LONG lW, LONG lH);
VOID    __stdcall RTP_SizeTextL(UINT uiID, HDC hDC, LONG* lpW, LONG* lpH);
LONG    __stdcall RTP_GetTextLength(UINT uiID);
VOID    __stdcall RTP_SelectAll(UINT uiID);
VOID	__stdcall RTP_SelectNone(UINT uiID);
VOID	__stdcall RTP_SelectRange(UINT uiID, long cpMin, long cpMax);
LONG	__stdcall RTP_GetSelectedTextLength(UINT uiID);
void	__stdcall RTP_GetRTFText(UINT uiID, BOOL bSelectionOnly, IStream** ppStream, ULONG* ulStrLen);
HRESULT __stdcall RTP_SetRTFText(UINT uiID, BOOL bSelectionOnly, LPSTR lpszRTFText);

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	if (ul_reason_for_call == DLL_PROCESS_ATTACH) {
		m_hModule = (HMODULE) hModule;
		for (int i = 0; i < MAX_RICH_TEXT; i++)
			m_pRTPArr[i] = NULL;
	}
	else if (ul_reason_for_call == DLL_PROCESS_DETACH) {
		for (int i = 0; i < MAX_RICH_TEXT; i++)
			if (m_pRTPArr[i])
				m_pRTPArr[i]->Release();
	}
    return TRUE;
}

UINT __stdcall RTP_Initialize()
{
	UINT uiID = -1;
	for (int i = 0; i < MAX_RICH_TEXT; i++) {
		if (m_pRTPArr[i] == NULL) {
			uiID = i;
			break;
		}
	}
	
	if (uiID != -1) {
		m_pRTPArr[uiID] = new X3RichTextPrinter();
		m_pRTPArr[uiID]->SetText(L"Xiphias3");
	}

	return uiID;
}

BOOL __stdcall RTP_Terminate(UINT uiID)
{
	if (m_pRTPArr[uiID]) {
		m_pRTPArr[uiID]->Release();
		m_pRTPArr[uiID] = NULL;
		return TRUE;
	}
	return FALSE;
}

VOID __stdcall RTP_SetText(UINT uiID, LPCWSTR lpwszText)
{
	if (m_pRTPArr[uiID])
		m_pRTPArr[uiID]->SetText(lpwszText);
}

LONG __stdcall RTP_GetTextLength(UINT uiID)
{
	if (m_pRTPArr[uiID]) {
		ITextRange* pRange;
		LONG len = 0;

		m_pRTPArr[uiID]->Range(0, -1, &pRange);
		pRange->GetStoryLength(&len);
		pRange->Release();
		return len;
	}
	return -1;
}

LONG __stdcall RTP_GetSelectedTextLength(UINT uiID)
{
	if (m_pRTPArr[uiID]) {
		CHARRANGE sCR;
		RTP_SendMessage(uiID, EM_EXGETSEL, NULL, (LPARAM) &sCR, NULL);
		return (sCR.cpMax - sCR.cpMin);
	}
	return -1;
}

VOID __stdcall RTP_SelectAll(UINT uiID)
{
	if (m_pRTPArr[uiID])
		RTP_SelectRange(uiID, 0, -1);
}

VOID __stdcall RTP_SelectNone(UINT uiID)
{
	if (m_pRTPArr[uiID])
		RTP_SelectRange(uiID, 0, 0);
}

VOID __stdcall RTP_SelectRange(UINT uiID, long cpMin, long cpMax)
{
	if (m_pRTPArr[uiID]) {
		CHARRANGE sCR;

		sCR.cpMin = cpMin;
		sCR.cpMax = cpMax;
		RTP_SendMessage(uiID, EM_EXSETSEL, NULL, (LPARAM) &sCR, NULL);
	}
}

VOID __stdcall RTP_SizeTextL(UINT uiID, HDC hDC, LONG* lpW, LONG* lpH)
{
	RECT rc = {0, 0, *lpW, *lpH};
	if (m_pRTPArr[uiID]) {
		RTP_SizeText(uiID, hDC, &rc);
		*lpW = rc.right;
		*lpH = rc.bottom;
	}
}

VOID __stdcall RTP_SizeText(UINT uiID, HDC hDC, RECT* rect)
{
	if (m_pRTPArr[uiID])
		m_pRTPArr[uiID]->SizeText(hDC, rect);
}

BOOL __stdcall RTP_DrawTextL(UINT uiID, HDC hDC, LONG lX, LONG lY, LONG lW, LONG lH)
{
	RECT rc = {lX, lY, (lX + lW), (lY + lH)};
	if (m_pRTPArr[uiID]) 
		return RTP_DrawText(uiID, hDC, &rc);
	return FALSE;
}

BOOL __stdcall RTP_DrawText(UINT uiID, HDC hDC, RECT* rect)
{
	if (m_pRTPArr[uiID]) 
		return m_pRTPArr[uiID]->DrawText(hDC, rect);
	return FALSE;
}

HRESULT __stdcall RTP_SendMessage(UINT uiID, UINT msg, WPARAM wparam, LPARAM lparam, LRESULT* plresult)
{
	if (m_pRTPArr[uiID])
		return m_pRTPArr[uiID]->SendMessage(msg, wparam, lparam, plresult);
	return E_FAIL;
}

void __stdcall RTP_GetRTFText(UINT uiID, BOOL bSelectionOnly, IStream** ppStream, ULONG* ulStrLen)
{
	if (m_pRTPArr[uiID]) {
		m_pRTPArr[uiID]->getRTFText(bSelectionOnly, ppStream, ulStrLen);
	}
}

HRESULT __stdcall RTP_SetRTFText(UINT uiID, BOOL bSelectionOnly, LPSTR lpszRTFText)
{
	if (m_pRTPArr[uiID]) {
		return m_pRTPArr[uiID]->setRTFText(bSelectionOnly, lpszRTFText);
	}
	return E_FAIL;
}